import enum
import logging

import datetime
from django_cron import CronJobBase, Schedule
from satella.instrumentation.metrics import getMetric

from counting.models import StoryOfADay

from shares.models import Share, SHARE_FILE

logger = logging.getLogger(__name__)
entries_compiled = getMetric('entries.compiled', 'counter')


class DayType(enum.IntEnum):
    # a day that has already been processed
    DAY_WITH_HISTORY = 0

    # a day whose links will have fully expired, that is not yet in history
    DAY_UNPROCESSED_BUT_PROCESSABLE = 1

    # a day for which the statistics need to be calculated in real time
    DAY_LIVE = 2


class ReaperJob(CronJobBase):
    """
    Reaper's job is to collect dead links within the database and compile a history out of them.
    It will delete only links from previous days, and only for days that have all of their
    links expired.
    """
    RUN_EVERY_MINS = 24 * 60  # once each day

    schedule = Schedule(run_every_mins=RUN_EVERY_MINS)
    code = 'reaper-job'  # a unique code

    @staticmethod
    def get_day_type(date: datetime.date) -> DayType:
        if date >= datetime.date.today() - datetime.timedelta(days=2):
            return DayType.DAY_LIVE

        try:
            StoryOfADay.objects.get(date=date)
            return DayType.DAY_WITH_HISTORY
        except StoryOfADay.DoesNotExist:
            # this is going to do the right thing even if the iterator's empty
            if all(item.expired for item in Share.objects.get_for_day(date)):
                return DayType.DAY_UNPROCESSED_BUT_PROCESSABLE
            else:
                return DayType.DAY_LIVE

    def do(self):
        try:
            cur_day = Share.objects.order_by('created_on')[0].creation_date
        except IndexError:
            return  # no links to process

        while self.get_day_type(cur_day) != DayType.LIVE:
            links, files = 0, 0
            for share in Share.objects.get_for_day(cur_day):
                if share.times_used:
                    if share.share_type == SHARE_FILE:
                        files += 1
                    else:
                        links += 1
            sod = StoryOfADay(day=cur_day, links=links, files=files)
            logger.info('Historic info for %s compiled, %s files visited, %s links visited',
                        cur_day, files, links)
            entries_compiled.runtime(+1)
            sod.save()
            share.delete()
            cur_day = cur_day + datetime.timedelta(days=1)
