import logging
from django.shortcuts import get_object_or_404
from django.http import StreamingHttpResponse as SHTTPResponse, HttpResponseRedirect
from rest_framework import serializers, status
from rest_framework.parsers import JSONParser, FileUploadParser, MultiPartParser
from rest_framework.response import Response
from rest_framework.views import APIView
from satella.instrumentation.metrics import getMetric

from shares.models import Share, hash_password, SHARE_FILE, add_file, add_url

logger = logging.getLogger(__name__)
files = getMetric('visited.api.files', 'counter')
links = getMetric('visited.api.links', 'counter')


class AddShareSerializer(serializers.Serializer):
    """this is for documentation autogeneration purpose only"""
    file = serializers.FileField(required=False)
    url = serializers.CharField(required=False)


class ShareSerializer(serializers.Serializer):
    url = serializers.CharField(required=True)
    password = serializers.CharField(required=True)


class AddShare(APIView):
    """
    Adds a share.

    You have to provide either a field called url or upload a file using multipart.
    You should name it file.
    The result will be a JSON object containing two properties: url and password.
    The result will be passed using HTTP 201.

    You must be authorized to access this endpoint.
    """
    parser_classes = JSONParser, MultiPartParser, FileUploadParser

    def get_serializer(self):
        return AddShareSerializer()

    def put(self, request):
        if not request.user.is_authenticated:
            return Response({}, status=status.HTTP_401_UNAUTHORIZED)
        if request.FILES.get('file'):
            dct = add_file(request.FILES['file'], request, escape_url=False)
            logger.info('Created a file via API')
        elif request.data.get('url'):
            dct = add_url(request.data['url'], request, escape_url=False)
            logger.info('Created an URL via API')
        else:
            return Response({}, status=status.HTTP_400_BAD_REQUEST)

        return Response(dct, status=status.HTTP_201_CREATED)


class GetShareSerializer(serializers.Serializer):
    password = serializers.CharField(required=True, label='Password')


class GetShare(APIView):
    """
    Loads the history

    One JSON argument is expected - password, to contain the password.
    You will receive either a 302 Redirect or a 200 OK with the file contents
    """

    def get_serializer(self, **kwargs):
        return GetShareSerializer(**kwargs)

    def post(self, request, share_id: str, format=None):
        serializer = self.get_serializer(data=request.data)
        if not serializer.is_valid():
            return Response({}, status=status.HTTP_400_BAD_REQUEST)

        share = get_object_or_404(Share, id=share_id)

        if share.expired:
            return Response({}, status=status.HTTP_404_NOT_FOUND)

        if hash_password(serializer.data['password']) != share.pwd_hash:
            return Response({}, status=status.HTTP_401_UNAUTHORIZED)

        try:
            if share.share_type == SHARE_FILE:
                files.runtime(+1)
                return SHTTPResponse(**share.get_kwargs_for_s_http_response())
            else:
                links.runtime(+1)
                return HttpResponseRedirect(redirect_to=share.resource)
        finally:
            share.times_used += 1
            share.save()
